@@ -4,6 +4,6 @@
 
 case "$DISPLAY" in
 	":0"|":0.0"|"$HOSTNAME:0.0")
-		/usr/bin/medusa-idled
+		/gnome/bin/medusa-idled
 	;;
 esac
