@@ -43,12 +43,19 @@
 #include <sys/types.h>
 #include <sys/un.h>
 #include <sys/vfs.h>
+
+#ifdef HAVE_STATVFS
+#include <sys/statvfs.h>
+#elif defined(HAVE_STATFS)
+#include <sys/statfs.h>
+#endif /* HAVE_STATVFS */
+
 #include <time.h>
 #include <unistd.h>
 
 #include <libmedusa-internal/medusa-conf.h>
 #include <libmedusa-internal/medusa-debug.h>
-#include <libmedusa-internal/medusa-lock.h>
+#include <libmedusa/medusa-lock.h>
 #include <libmedusa-internal/medusa-master-db.h>
 #include <libmedusa/medusa-search-service-private.h>
 #include <libmedusa/medusa-index-progress.h>
@@ -60,10 +67,6 @@
 #define AF_LOCAL AF_UNIX
 #endif
 
-#ifndef NAME_MAX
-#define NAME_MAX 512
-#endif
-
 #define INDEX_SERVICE_LISTENER_DEBUG
 
 #ifndef SUN_LEN
@@ -310,24 +313,46 @@ do_index_accounting (void)
 static void
 exit_if_disk_is_too_full ()
 {
-        struct statfs statfs_buffer;
+#ifdef HAVE_STATVFS
+        struct statvfs stat_buffer;
+#elif defined(HAVE_STATFS)
+        struct statfs stat_buffer;
+#else 
+#warning "You don't have statfs or statfs installed on your system.  The indexer will not be able to determine "
+        "if your disk is too full for indexing"
+#endif
         char *error_message;
-        int statfs_result;
+        int stat_result;
+        unsigned long number_of_blocks, number_of_free_blocks, block_size;
         
-        statfs_result = statfs (MEDUSA_PREFIX INDEX_PATH, &statfs_buffer);                
-        if (statfs_result == -1) {
-                error_message = g_strdup_printf ("Can't determine file system status for %s, so the disk may be too full for an index. Indexing is continuing", MEDUSA_PREFIX INDEX_PATH);
+#ifdef HAVE_STATVFS
+        stat_result = statvfs (MEDUSA_LOCALSTATEDIR, &stat_buffer); 
+#elif defined(HAVE_STATFS)
+        stat_result = statfs (MEDUSA_LOCALSTATEDIR, &stat_buffer); 
+#endif
+
+        if (stat_result == -1) {
+                error_message = g_strdup_printf ("Can't determine file system "
+			"status for %s, so the disk may be too full for an "
+			"index. Indexing is continuing", MEDUSA_LOCALSTATEDIR);
                 medusa_log_error (error_message);
                 g_free (error_message);
                 return;
         }
+
+        number_of_blocks = (unsigned long) stat_buffer.f_blocks;
+        number_of_free_blocks = (unsigned long) stat_buffer.f_bfree;
+        block_size = (unsigned long) stat_buffer.f_bsize;
+
         g_print ("File system %s has %ld blocks, %ld free\n", 
-                 MEDUSA_PREFIX INDEX_PATH, statfs_buffer.f_blocks, statfs_buffer.f_bfree);
+                 MEDUSA_LOCALSTATEDIR, number_of_blocks, number_of_free_blocks);
         /* We'll need 40 megs or so for a conservative index estimate */
-        if ((statfs_buffer.f_blocks / statfs_buffer.f_bfree) > 20 ||
-            statfs_buffer.f_bfree * statfs_buffer.f_bsize < (40<<20)) {
-                error_message = g_strdup_printf ("Indexing aborted because disk is either more than 95%% full or 40 megs of space are not available at location %s",
-                                                 MEDUSA_PREFIX INDEX_PATH);
+        if ((number_of_blocks / number_of_free_blocks) > 20 ||
+            number_of_free_blocks * block_size < (40<<20)) {
+                error_message = g_strdup_printf ("Indexing aborted because "
+			"disk is either more than 95%% full or 40 megs of "
+			"space are not available at location %s",
+                                                 MEDUSA_LOCALSTATEDIR);
                 medusa_log_error (error_message);
                 g_free (error_message);
                 exit (1);
