@@ -109,7 +109,7 @@ medusa_file_index_is_file (gpointer database,
         file_system_db = (MedusaFileSystemDB *) database;
         database_record = (MedusaRDBRecord ) record;
 
-        mime_type_in_record = g_new0 (char, NAME_MAX);
+        mime_type_in_record = g_new0 (char, FILE_NAME_MAXIMUM_LENGTH);
         medusa_rdb_record_get_field_value (database_record,
                                            medusa_file_system_db_get_field_info (file_system_db),
                                            MEDUSA_FILE_INDEX_MIME_TYPE_FIELD_TITLE,
@@ -134,7 +134,7 @@ medusa_file_index_is_text_file (gpointer database,
         file_system_db = (MedusaFileSystemDB *) database;
         database_record = (MedusaRDBRecord ) record;
 
-        mime_type_in_record = g_new0 (char, NAME_MAX);
+        mime_type_in_record = g_new0 (char, FILE_NAME_MAXIMUM_LENGTH);
         medusa_rdb_record_get_field_value (database_record,
                                            medusa_file_system_db_get_field_info (file_system_db),
                                            MEDUSA_FILE_INDEX_MIME_TYPE_FIELD_TITLE,
@@ -158,7 +158,7 @@ medusa_file_index_is_application (gpointer database,
         file_system_db = (MedusaFileSystemDB *) database;
         database_record = (MedusaRDBRecord ) record;
 
-        mime_type_in_record = g_new0 (char, NAME_MAX);
+        mime_type_in_record = g_new0 (char, FILE_NAME_MAXIMUM_LENGTH);
         medusa_rdb_record_get_field_value (database_record,
                                            medusa_file_system_db_get_field_info (file_system_db),
                                            MEDUSA_FILE_INDEX_MIME_TYPE_FIELD_TITLE,
@@ -182,7 +182,7 @@ medusa_file_index_is_music (gpointer database,
         file_system_db = (MedusaFileSystemDB *) database;
         database_record = (MedusaRDBRecord ) record;
 
-        mime_type_in_record = g_new0 (char, NAME_MAX);
+        mime_type_in_record = g_new0 (char, FILE_NAME_MAXIMUM_LENGTH);
         medusa_rdb_record_get_field_value (database_record,
                                            medusa_file_system_db_get_field_info (file_system_db),
                                            MEDUSA_FILE_INDEX_MIME_TYPE_FIELD_TITLE,
@@ -206,7 +206,7 @@ medusa_file_index_is_directory (gpointer database,
         file_system_db = (MedusaFileSystemDB *) database;
         database_record = (MedusaRDBRecord ) record;
 
-        mime_type_in_record = g_new0 (char, NAME_MAX);
+        mime_type_in_record = g_new0 (char, FILE_NAME_MAXIMUM_LENGTH);
         medusa_rdb_record_get_field_value (database_record,
                                            medusa_file_system_db_get_field_info (file_system_db),
                                            MEDUSA_FILE_INDEX_MIME_TYPE_FIELD_TITLE,
@@ -330,6 +330,78 @@ medusa_file_index_is_modified_after_time (gpointer database,
        
 }
 
+gboolean
+medusa_file_index_is_not_modified_on_date (gpointer database,
+                                           gpointer record,
+                                           MedusaQueryArgument argument)
+{
+        MedusaFileSystemDB *file_system_db;
+        MedusaRDBRecord database_record;
+        char *date_in_query;
+        static char *last_date_in_query = NULL;
+        static time_t first_time_corresponding_to_date;
+        static time_t last_time_corresponding_to_date;
+        time_t time_in_record;
+
+        file_system_db = (MedusaFileSystemDB *) database;
+        database_record = (MedusaRDBRecord) record;
+        date_in_query = argument.string;
+        
+        if (!last_date_in_query ||
+            strcmp (last_date_in_query, date_in_query) != 0) {
+                g_free (last_date_in_query);
+                last_date_in_query = g_strdup (argument.string);
+                first_time_corresponding_to_date = medusa_file_info_get_first_unix_time_occurring_on_date (date_in_query);
+                last_time_corresponding_to_date = medusa_file_info_get_last_unix_time_occurring_on_date (date_in_query);
+        }
+
+        medusa_rdb_record_get_field_value (database_record,
+                                           medusa_file_system_db_get_field_info (file_system_db),
+                                           MEDUSA_FILE_INDEX_MTIME_FIELD_TITLE,
+                                           file_system_db,
+                                           &time_in_record);
+
+        return (time_in_record > last_time_corresponding_to_date ||
+                time_in_record < first_time_corresponding_to_date);
+}
+
+gboolean 
+medusa_file_index_verify_file_is_not_modified_on_date (const char *uri,
+                                                       MedusaQueryArgument argument)
+{
+        GnomeVFSFileInfo file_info;
+        GnomeVFSResult get_info_result;
+        char *date_in_query;
+        static char *last_date_in_query = NULL;
+        static time_t first_time_corresponding_to_date;
+        static time_t last_time_corresponding_to_date;
+
+        date_in_query = argument.string;
+
+        if (!last_date_in_query ||
+            strcmp (last_date_in_query, date_in_query) != 0) {
+                g_free (last_date_in_query);
+                last_date_in_query = g_strdup (argument.string);
+                first_time_corresponding_to_date = medusa_file_info_get_first_unix_time_occurring_on_date (date_in_query);
+                last_time_corresponding_to_date = medusa_file_info_get_last_unix_time_occurring_on_date (date_in_query);
+        }
+
+
+        get_info_result = gnome_vfs_get_file_info (uri,
+                                                   &file_info,
+                                                   GNOME_VFS_FILE_INFO_DEFAULT);
+
+        /* If something happened to the file, don't return it */
+        if (get_info_result != GNOME_VFS_OK) {
+                return FALSE;
+        }
+        else {
+                return file_info.mtime < first_time_corresponding_to_date ||
+                        file_info.mtime > last_time_corresponding_to_date;;
+        }
+}
+
+
 gboolean
 medusa_file_index_is_larger_than (gpointer database,
                                   gpointer record,
@@ -467,7 +539,7 @@ medusa_file_index_marked_with_keyword (gpointer data,
                                        MedusaQueryArgument keyword_argument)
 {
         char *keyword;
-        char keywords_string[NAME_MAX];
+        char keywords_string[FILE_NAME_MAXIMUM_LENGTH];
         MedusaDatabaseAndUIDData *database_and_uid;
         MedusaKeywordSet *keyword_set;
         MedusaFileSystemDB *file_system_db;
