@@ -104,7 +104,9 @@ medusa_rdb_file_open (const char *file_name,
         file->field_info = medusa_rdb_field_info_new ();
         file->metainfo_length = metainfo_length;
 
-        g_return_val_if_fail (access (file_name, R_OK | W_OK) == 0, NULL);
+        if (access (file_name, R_OK | W_OK) != 0) {
+                return NULL;
+        }
         file->io_handler = medusa_io_handler_open (file_name,
                                                    MEDUSA_RDB_FILE_MAGIC_NUMBER,
                                                    MEDUSA_RDB_FILE_VERSION);
