@@ -31,17 +31,21 @@
 #include "medusa-search-service-private.h"
 #include "medusa-service-private.h"
 #include "medusa-string.h"
+#include "medusa-system-state.h"
 #include "medusa-utils.h"
+#include <errno.h>
 #include <glib.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/types.h>
+#include <sys/time.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <libgnomevfs/gnome-vfs-types.h>
+#include <libgnomevfs/gnome-vfs-context.h>
+#include <libgnomevfs/gnome-vfs-cancellation.h>
 #undef DEBUG_INDEXED_SEARCH_SERVICE
 #ifdef DEBUG_INDEXED_SEARCH_SERVICE
 #include <libgnomevfs/gnome-vfs-result.h>
@@ -59,6 +63,23 @@ struct MedusaIndexedSearch {
 };
 
 
+static int
+read_cancellable (int file_descriptor, void *buffer,
+                  size_t size, GnomeVFSContext *context)
+{
+        int read_length;
+
+        do {
+                read_length = read (file_descriptor, buffer, size);
+                usleep (200);
+        } while (read_length == -1
+                 && errno == EAGAIN
+                 && !gnome_vfs_context_check_cancellation (context));        
+
+        return read_length;
+}
+
+
 static char *
 get_search_socket_name (void)
 {
@@ -91,13 +112,15 @@ authenticate_connection (MedusaIndexedSearch *connection) {
         if (write_result == -1) {
                 return GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE;
         }
-        
-        read_result = read (connection->socket_fd, acknowledgement_buffer, 12);
+
+        read_result = read_cancellable (connection->socket_fd, acknowledgement_buffer, 12, NULL);
+
         if (read_result == -1) {
                 return GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE;
         }
 
-        if (medusa_str_has_prefix (acknowledgement_buffer, SEARCH_INDEX_ERROR_TRANSMISSION)) {
+        if (medusa_str_has_prefix (acknowledgement_buffer, SEARCH_INDEX_ERROR_TRANSMISSION) ||
+            medusa_str_has_prefix (acknowledgement_buffer, SEARCH_REQUEST_TIMEOUT_ERROR)) {
                 return GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE;
         }
 
@@ -114,6 +137,7 @@ authenticate_connection (MedusaIndexedSearch *connection) {
         /* Keep looking if cookie file isn't created yet */
         while (cookie_fd == -1) {
                 cookie_fd = open (file_name, O_RDONLY);
+                usleep (200);
         }
         
         g_free (file_name);
@@ -134,19 +158,26 @@ initialize_connection_buffer (MedusaIndexedSearch *connection)
         connection->buffer = g_strdup ("");
 }
 
-static void
-refresh_read_buffer_if_needed (MedusaIndexedSearch *connection)
+
+
+static gboolean
+refresh_read_buffer_if_needed (MedusaIndexedSearch *connection,
+                               GnomeVFSContext *context)
 {
         char tmpbuf[513];
         char *new_buffer;
         int len;
+        
+
+        g_assert (connection->buffer != NULL);
 
         while (strchr (connection->buffer, '\n') == NULL) {
-                len = read (connection->socket_fd, &tmpbuf, 512 * sizeof (char)); 
+                len = read_cancellable (connection->socket_fd, &tmpbuf, 512 * sizeof (char),
+                                        context);
                 if (len == -1) {
                         g_free (connection->buffer);
                         connection->buffer = NULL;
-                        return;
+                        return FALSE;
                 }
 #ifdef DEBUG_SEARCH_API
                 g_print ("read from socket %s: %d bytes\n", tmpbuf, len);
@@ -158,22 +189,22 @@ refresh_read_buffer_if_needed (MedusaIndexedSearch *connection)
                 connection->buffer = new_buffer;
         }
 
-
+        return TRUE;
 }
 
 static char *
-read_connection_line (MedusaIndexedSearch *connection)
+read_connection_line (MedusaIndexedSearch *connection,
+                      GnomeVFSContext *context)
 {
         const char *newline_pos;
         char *result_uri;
         char *new_buffer;
 
-        refresh_read_buffer_if_needed (connection);
-
-        /* grab a line */
-        if (connection->buffer == NULL) {
+        if (refresh_read_buffer_if_needed (connection,
+                                           context) == FALSE) {
                 return NULL;
         }
+
         newline_pos = strchr (connection->buffer, '\n');
         g_assert (newline_pos != NULL);
         result_uri = g_strndup (connection->buffer, newline_pos - connection->buffer);
@@ -197,7 +228,8 @@ get_search_uri_parsing_result (MedusaIndexedSearch *connection)
 {
         char *search_daemon_return;
 
-        search_daemon_return = read_connection_line (connection);
+        search_daemon_return = read_connection_line (connection,
+                                                     NULL);
         /* Connection was closed */
         if (search_daemon_return == NULL) {
                 return GNOME_VFS_ERROR_INTERRUPTED;
@@ -227,18 +259,18 @@ get_search_uri_parsing_result (MedusaIndexedSearch *connection)
 GnomeVFSResult
 medusa_indexed_search_is_available ()
 {
-        GnomeVFSResult new_search_result;
-        MedusaIndexedSearch *connection;
+        char *search_socket_path;
+        int socket_fd;
 
-        connection = medusa_indexed_search_new (&new_search_result, "search:index-only[file:///]test search");
-#ifdef DEBUG_SEARCH_API
-        g_print ("New connection result was %s\n", gnome_vfs_result_to_string (new_search_result));
-#endif
-        medusa_indexed_search_destroy (connection);
-        if (new_search_result == GNOME_VFS_ERROR_INVALID_URI) {
+        search_socket_path = get_search_socket_name ();
+        socket_fd = medusa_initialize_socket_for_requests (search_socket_path);
+        g_free (search_socket_path);
+        if (socket_fd == -1) {
+                return GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE;
+        }
+        else {
                 return GNOME_VFS_OK;
         }
-        return new_search_result;
 }
 
 gboolean
@@ -251,8 +283,10 @@ medusa_indexed_search_system_index_files_look_available (void)
            available, not just two. We should also have a place
            to get the list of index file names from somewhere, rather than
            having a second copy of the name here.  */
-        uri_list_filename = g_strdup_printf ("%s%s%s", MEDUSA_PREFIX, "/share/medusa/", "uri-list");
-        text_index_location_filename = g_strdup_printf ("%s%s%s", MEDUSA_PREFIX, "/share/medusa/", "text-index-location-file");
+        uri_list_filename = g_strdup_printf ("%s/%s", MEDUSA_LOCALSTATEDIR,
+					     "uri-list");
+        text_index_location_filename = g_strdup_printf ("%s/%s",
+			MEDUSA_LOCALSTATEDIR, "text-index-location-file");
 
         index_files_exist = (access (uri_list_filename, F_OK) == 0) &&
                 (access (text_index_location_filename, F_OK) == 0);
@@ -285,15 +319,23 @@ send_search_uri (MedusaIndexedSearch *connection,
 
 MedusaIndexedSearch *
 medusa_indexed_search_new (GnomeVFSResult *result,
+                           GnomeVFSContext *context,
                            const char *uri)
 {
         MedusaIndexedSearch *connection;
         char *search_socket_path;
 
+        if (medusa_system_services_are_enabled () == FALSE) {
+                *result = GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE;
+                return NULL;
+        }
+        
         connection = g_new0 (MedusaIndexedSearch, 1);
 
         search_socket_path = get_search_socket_name ();
         connection->socket_fd = medusa_initialize_socket_for_requests (search_socket_path);        
+        /* Set this to nonblocking so that we can check for cancellation */
+        fcntl (connection->socket_fd, F_SETFL, O_NONBLOCK);
         g_free (search_socket_path);
 
 
@@ -317,8 +359,12 @@ medusa_indexed_search_new (GnomeVFSResult *result,
         }
         initialize_connection_buffer (connection);
         send_search_uri (connection, uri);
-        refresh_read_buffer_if_needed (connection);
-
+        if (refresh_read_buffer_if_needed (connection,
+                                           context) == FALSE) {
+                g_free (connection);
+                *result = GNOME_VFS_ERROR_INTERRUPTED;
+                return NULL;
+        }        
         if (*result == GNOME_VFS_OK) {
                 *result = get_search_uri_parsing_result (connection);
 #ifdef DEBUG_MEDUSA_INDEXED_SEARCH_SERVICE
@@ -353,6 +399,7 @@ medusa_indexed_search_start_search (MedusaIndexedSearch *connection,
 
 GnomeVFSResult 
 medusa_indexed_search_read_search_result (MedusaIndexedSearch *connection,
+                                          GnomeVFSContext *context,
                                           char **result)
 {
         char *result_uri;
@@ -368,8 +415,12 @@ medusa_indexed_search_read_search_result (MedusaIndexedSearch *connection,
                 return GNOME_VFS_ERROR_EOF;
         }
 
-        result_uri = read_connection_line (connection);
-        g_return_val_if_fail (result_uri != NULL, GNOME_VFS_ERROR_NOT_FOUND);
+        result_uri = read_connection_line (connection,
+                                           context);
+
+        if (result_uri == NULL) {
+                return GNOME_VFS_ERROR_CANCELLED;
+        }
 
         if (strcmp (result_uri, SEARCH_END_TRANSMISSION) == 0) {
                 g_free (result_uri);
