@@ -34,6 +34,7 @@
 #include <glib.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-types.h>
+#include <libgnomevfs/gnome-vfs-cancellation.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-context.h>
 #include <regex.h>
@@ -72,13 +73,14 @@ struct MedusaUnindexedSearch {
         GList *keywords_criteria;
 
         /* Root directory for the search */
-        char *root_directory;
+        char *root_uri;
 
-        /* Place to put results */
-        char *output_file_name;
-        int output_fd;
-        int read_fd;
-        char *buffer;
+        /* "Stack" of directory stream pointers, so we remember where
+           we were reading in order to get the next search result */
+        GList *directory_name_stack;
+        GList *directory_stream_stack;
+        /* Also keep a "stack" of keyword hash tables */
+        GList *keywords_stack;
 
         /* Keep a boolean for a bad / illogical search,
            so we know not to bother looking */
@@ -90,17 +92,27 @@ struct MedusaUnindexedSearch {
         gboolean search_discovers_file_type;
         
         gboolean search_is_running;
-        
 };
 
+typedef struct {
+        gboolean directory_has_keywords;
+        GHashTable *hash_table;
+} DirectoryKeywords;
 
-static char *                  get_root_directory_from_uri    (const char *uri);
+static char *                  unbracket_root_uri             (const char *seach_uri_without_scheme_or_method);
 static char *                  uri_to_search_string           (const char *uri);
 static void                    assume_file_is_directory_unless_a_link (const char *uri,
                                                                        gboolean *file_is_directory);
 static GList *                 search_string_to_criteria      (char *search_string);
-static void                    search_directory               (char *directory, 
-                                                               MedusaUnindexedSearch *search);
+static char *                  search_directory               (MedusaUnindexedSearch *search,
+                                                               const char *directory_uri,
+                                                               GnomeVFSContext *context,
+                                                               gboolean *search_was_cancelled);
+static char *                  search_directory_contents_in_progress (MedusaUnindexedSearch *search,
+                                                                      GnomeVFSContext *context,
+                                                                      gboolean *search_was_cancelled);
+static void                    keywords_information_destroy          (DirectoryKeywords *keywords_information);
+static void                    pop_keyword_information_and_directory_from_stacks (MedusaUnindexedSearch *search);
 static gboolean                file_matches_criteria          (const char *file_uri,
                                                                MedusaUnindexedSearch *search,
                                                                gboolean *file_is_directory,
@@ -159,8 +171,6 @@ static gboolean                criterion_is_keywords_criterion(gpointer criterio
                                                                gpointer user_data);
 static gboolean                criterion_field_is             (const char *criterion,
                                                                const char *field_name);   
-static int                     initialize_results_file_for_writing  (char *file_name);
-static int                     initialize_results_file_for_reading  (char *file_name);
 static char *                  match_string_to_date           (const char *match_string);
 
 static const char *            second_word_of                 (const char *criterion);
@@ -168,19 +178,19 @@ static const char *            last_word_of                   (const char *crite
 
 MedusaUnindexedSearch *
 medusa_unindexed_search_new (GnomeVFSResult *result,
-                             const char *uri)
+                             const char *search_uri_without_scheme_or_method)
 {
         char *search_string;
         GList *other_criteria;
         MedusaUnindexedSearch *search;
   
-        if (medusa_unindexed_search_is_available_for_uri (uri) != GNOME_VFS_OK) {
+        if (medusa_unindexed_search_is_available_for_uri (search_uri_without_scheme_or_method) != GNOME_VFS_OK) {
                 *result = GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE;
                 return NULL;
         }
         search = g_new0 (MedusaUnindexedSearch, 1);
-        search->root_directory = get_root_directory_from_uri (uri);
-        search_string = uri_to_search_string (uri);
+        search->root_uri = unbracket_root_uri (search_uri_without_scheme_or_method);
+        search_string = uri_to_search_string (search_uri_without_scheme_or_method);
         /* Avoid case issues now */
         g_strdown (search_string);
         search->criteria = search_string_to_criteria (search_string);
@@ -212,16 +222,11 @@ medusa_unindexed_search_new (GnomeVFSResult *result,
            if it contains criteria we don't know about */
         search->invalid_search = (g_list_length (other_criteria) > 0);
         medusa_g_list_free_deep (other_criteria);
-
-        /* Initialize other side of the search connection to
-           write results to */
-        search->output_file_name = g_strdup_printf ("%s.%lu", UNINDEXED_RESULTS_FILE, (unsigned long) getpid ());
-        search->output_fd = initialize_results_file_for_writing (search->output_file_name);
-        search->read_fd = initialize_results_file_for_reading (search->output_file_name);
-        if (search->read_fd == -1 || search->output_fd == -1) {
-                *result = GNOME_VFS_ERROR_INTERNAL;
+        
+        if (search->invalid_search) {
+                *result = GNOME_VFS_ERROR_INVALID_URI;
         }
-        else {
+        else { 
                 *result = GNOME_VFS_OK;
         }
 
@@ -241,84 +246,73 @@ medusa_unindexed_search_is_available_for_uri (const char *uri)
         return GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE;
 }
 
-GnomeVFSResult
-medusa_unindexed_search_start_search (MedusaUnindexedSearch *search)
-
+static void
+cleanup_current_directory_information (MedusaUnindexedSearch *search)
 {
-        pid_t result_process;
+        closedir (search->directory_stream_stack->data);
+        keywords_information_destroy (search->keywords_stack->data);
+        g_free (search->directory_name_stack->data);
         
-        if (search->invalid_search) {
-                return GNOME_VFS_ERROR_INVALID_URI;
-        }
-        result_process = fork ();
-        if (result_process == 0) {
-                search_directory (search->root_directory, search);
-#ifdef MEDUSA_UNINDEXED_SEARCH_DEBUG
-                printf ("Search process is complete\n");
-#endif
-                write (search->output_fd, 
-                       END_OF_UNINDEXED_SEARCH_RESULTS,
-                       strlen (END_OF_UNINDEXED_SEARCH_RESULTS));
-                medusa_unindexed_search_destroy (search);
-                exit (0);
-        }
-        close (search->output_fd);
-        return GNOME_VFS_OK;
+        pop_keyword_information_and_directory_from_stacks (search);
+
 }
 
 GnomeVFSResult         
 medusa_unindexed_search_read_search_result (MedusaUnindexedSearch *search,
                                             GnomeVFSContext *context,
-                                            char **result)
+                                            char **result_uri)
 { 
-        char tmpbuf[513];
-        const char *newline_pos;
-        char *result_uri;
-        int len;
-        char *buffer;
+        gboolean search_was_cancelled;
 
-        g_return_val_if_fail (search != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+        *result_uri = NULL;
+        search_was_cancelled = FALSE;
 
-        if (search->buffer == NULL) {
-                search->buffer = g_strdup ("");
+        if (search->directory_stream_stack == NULL) {
+                *result_uri = search_directory (search, search->root_uri, context, &search_was_cancelled);
         }
-
-        while (strchr (search->buffer, '\n') == NULL) {
-                if (gnome_vfs_context_check_cancellation (context)) {
-                        return GNOME_VFS_ERROR_INTERRUPTED;
-                }
-                memset (&tmpbuf, 0, 513);
-                len = read (search->read_fd, &tmpbuf, 512 * sizeof (char)); 
-                if (len == 0) {
-                        sleep (1);
-                }
-                else if (len == -1) {
-                        *result = NULL;
-                        return GNOME_VFS_ERROR_EOF;
-                }
-                else {
-                        buffer = g_strconcat (search->buffer, tmpbuf, NULL);
-                        g_free (search->buffer);
-                        search->buffer = buffer;
-                }
+        else {
+                *result_uri = search_directory_contents_in_progress (search, context, &search_was_cancelled);
         }
-
         
-        /* grab a result */
-        newline_pos = strchr (search->buffer, '\n');
-        result_uri = g_strndup (search->buffer, newline_pos - search->buffer);
-        g_return_val_if_fail (result_uri != NULL, GNOME_VFS_ERROR_NOT_FOUND);
+        if (search_was_cancelled) {
+                return GNOME_VFS_ERROR_CANCELLED;
+        }
 
-        /* move buffer forward */
-        buffer = g_strdup (newline_pos + 1);
-        g_free (search->buffer);
-        search->buffer = buffer;
-
-        if (strcmp (result_uri, END_OF_UNINDEXED_SEARCH_RESULTS) == 0) {
+        /* If we didn't find anything in the current directory,
+           go up the stack frame until we find a place with results,
+           or no more results */
+        cleanup_current_directory_information (search);
+        while (*result_uri == NULL &&
+               search->directory_name_stack) {
+                *result_uri = search_directory_contents_in_progress (search, context, &search_was_cancelled);
+                cleanup_current_directory_information (search);
+                if (search_was_cancelled) {
+                        return GNOME_VFS_ERROR_CANCELLED;
+                }
+        }
+        
+        if (*result_uri == NULL) {
                 return GNOME_VFS_ERROR_EOF;
         }
-        *result = result_uri;
-        return GNOME_VFS_OK;
+        else {
+                return GNOME_VFS_OK;
+        }
+}
+
+static void
+closedir_cover (gpointer node_data,
+                gpointer extra_data)
+{
+        closedir (node_data);
+}
+
+static void
+keywords_information_destroy_cover (gpointer node_data,
+                                    gpointer extra_data)
+{
+        g_assert (node_data != NULL);
+
+        keywords_information_destroy (node_data);
 }
 
 void                   
@@ -327,103 +321,167 @@ medusa_unindexed_search_destroy (MedusaUnindexedSearch *search)
         if (search == NULL) {
                 return;
         }
-        medusa_unsearched_locations_shutdown ();
-        close (search->read_fd);
-        unlink (search->output_file_name);
+
+        /* Should free unsearched locations here */
         medusa_g_list_free_deep (search->name_criteria);
         medusa_g_list_free_deep (search->inode_criteria);
         medusa_g_list_free_deep (search->mime_type_criteria);
         medusa_g_list_free_deep (search->keywords_criteria);
-        g_free (search->root_directory);
+        
+        medusa_g_list_free_deep_custom (search->directory_stream_stack, closedir_cover, NULL);
+        medusa_g_list_free_deep_custom (search->keywords_stack, keywords_information_destroy_cover, NULL);
+        medusa_g_list_free_deep (search->directory_name_stack);
+
+        g_free (search->root_uri);
         g_free (search);
 }
 
 
-static void                    
-search_directory (char *directory_name, 
-                  MedusaUnindexedSearch *search)
+static GList *
+g_list_remove_first_link (GList *list)
+{
+        return g_list_remove (list, list->data);
+}
+
+static void
+pop_keyword_information_and_directory_from_stacks (MedusaUnindexedSearch *search)
+{
+        search->directory_stream_stack = g_list_remove_first_link (search->directory_stream_stack);
+        search->directory_name_stack = g_list_remove_first_link (search->directory_name_stack);
+        search->keywords_stack = g_list_remove_first_link (search->keywords_stack);
+}
+
+static char *
+search_directory_contents_in_progress (MedusaUnindexedSearch *search,
+                                       GnomeVFSContext *context,
+                                       gboolean *search_was_cancelled)
 {
-        DIR *directory_table;
-        struct dirent *directory_entry;
-        char *plain_directory_name;
-        char *file_uri;
         gboolean file_is_directory;
-        ssize_t write_result;
-        GnomeVFSResult keywords_hash_table_result;
-        gboolean directory_has_keywords;
-        GHashTable *keywords_hash_table;
+        char *file_path;
+        char *file_uri, *file_uri_child_result;
+        struct dirent *directory_entry;
 
-        /* FIXME bugzilla.eazel.com 3006:  
-           This doesn't work to prevent searches 
-           of files that aren't directories */
-        if (medusa_is_unsearched_location (directory_name)) {
-                return;
-        }
+        DIR *directory_stream;
+        DirectoryKeywords *keyword_information;
+        char *directory_path;
 
-        plain_directory_name = gnome_vfs_get_local_path_from_uri (directory_name);
-        g_return_if_fail (plain_directory_name != NULL);
-	directory_table = opendir (plain_directory_name);
-	/* If the directory is unreadable, don't bother */
-        if (directory_table == NULL) {
-                /* This could be a whole slew of problems,
-                   from not being able to read it to the directory
-                   having being erased before we got here.  Let's
-                   just ditch */
-                g_free (plain_directory_name);
-                return ;
-        }
-        if (search->keywords_criteria != NULL) {
-                /* We need to collect the whole directory's
-                   keywords before going through the directory */
-                keywords_hash_table_result =
-                        medusa_keyword_hash_table_create_for_directory (&keywords_hash_table,
-                                                                        directory_name);
-                directory_has_keywords = keywords_hash_table_result == GNOME_VFS_OK;
-        }
-        else {
-                directory_has_keywords = FALSE;
-        }
+        g_assert (search->directory_stream_stack != NULL);
+        g_assert (search->keywords_stack != NULL);
+        g_assert (search->directory_name_stack != NULL);
 
-	while ((directory_entry = readdir (directory_table)) != NULL) {
+        directory_stream = search->directory_stream_stack->data;
+        keyword_information = search->keywords_stack->data;
+        directory_path = search->directory_name_stack->data;
+
+	while ((directory_entry = readdir (directory_stream)) != NULL) {
 		if (!strcmp (directory_entry->d_name,".") || 
 		    !strcmp (directory_entry->d_name,"..")) {
 			continue;
 		}
-		file_uri = medusa_full_uri_from_directory_and_file (directory_name,
-                                                                    directory_entry->d_name);
+
+                if (gnome_vfs_context_check_cancellation (context)) {
+                        g_warning ("Search was cancelled\n");
+                        *search_was_cancelled = TRUE;
+                        return NULL;
+                }
+
+		file_path = medusa_full_path_from_directory_and_file_name (directory_path,
+                                                                           directory_entry->d_name);
+                file_uri = gnome_vfs_get_uri_from_local_path (file_path);
+
+                keyword_information = search->keywords_stack->data;
                 if (!medusa_is_unsearched_location (file_uri) &&
                     file_matches_criteria (file_uri, 
                                            search, 
                                            &file_is_directory,
-                                           keywords_hash_table,
-                                           directory_has_keywords)) {
-#ifdef UNINDEXED_SEARCH_DEBUG
-                        printf ("writing uri %s to the file\n", file_uri);
-#endif
-                        
-                        write_result = write (search->output_fd, file_uri, strlen (file_uri));
-                        
-                        if (write_result <= 0) {
-                                /* FIXME bugzilla.eazel.com 3007: 
-                                   We need to determine
-                                   the correct error behavior in this case */
-                                exit (1);
-                        }
-                        write_result = write (search->output_fd, "\n", strlen ("\n"));
+                                           keyword_information->hash_table,
+                                           keyword_information->directory_has_keywords)) {
+                        g_free (file_path);
+                        return file_uri;
                 }
-                /* Don't follow links because we don't want to get 
-                   stuck in circular ones */
+
                 if (file_is_directory) {
-                        search_directory (file_uri, search);
+                        file_uri_child_result = search_directory (search, file_uri, context, search_was_cancelled);
+                        if (*search_was_cancelled ||
+                            file_uri_child_result != NULL) {
+                                return file_uri_child_result;
+                        }
                 }
                 g_free (file_uri);
         }
-        closedir (directory_table);
-        if (directory_has_keywords) {
-                medusa_keyword_hash_table_destroy (keywords_hash_table);
+
+        return NULL;
+}
+
+static void
+keywords_information_destroy (DirectoryKeywords *keyword_information)
+{
+        if (keyword_information->directory_has_keywords) {
+                medusa_keyword_hash_table_destroy (keyword_information->hash_table);
         }
         
-        g_free (plain_directory_name);
+        g_free (keyword_information);
+
+}
+
+static char *
+search_directory (MedusaUnindexedSearch *search, 
+                  const char *directory_uri,
+                  GnomeVFSContext *context,
+                  gboolean *search_was_cancelled)
+{
+        DIR *directory_stream;
+        char *directory_path;
+        GnomeVFSResult keywords_hash_table_result;
+        DirectoryKeywords *keyword_information;
+        char *search_result;
+
+        if (medusa_is_unsearched_location (directory_uri)) {
+                return NULL;
+        }
+        
+
+        directory_path = gnome_vfs_get_local_path_from_uri (directory_uri);
+        g_return_val_if_fail (directory_path != NULL, NULL);
+	directory_stream = opendir (directory_path);
+
+	/* If the directory is unreadable, don't bother */
+        if (directory_stream == NULL) {
+                g_free (directory_path);
+                return NULL;
+        }
+        search->directory_name_stack = g_list_prepend (search->directory_name_stack,
+                                                       directory_path);
+
+        search->directory_stream_stack = g_list_prepend (search->directory_stream_stack,
+                                                         directory_stream);
+
+        keyword_information = g_new0 (DirectoryKeywords, 1);
+        if (search->keywords_criteria != NULL) {
+                /* We need to collect the whole directory's
+                   keywords before going through the directory */
+                keywords_hash_table_result =
+                        medusa_keyword_hash_table_create_for_directory (&keyword_information->hash_table,
+                                                                        directory_uri);
+                keyword_information->directory_has_keywords = keywords_hash_table_result == GNOME_VFS_OK;
+        }
+        else {
+                keyword_information->directory_has_keywords = FALSE;
+        }
+        
+        search->keywords_stack = g_list_prepend (search->keywords_stack,
+                                                 keyword_information);
+
+        search_result = search_directory_contents_in_progress (search, context, search_was_cancelled);
+
+        if (*search_was_cancelled ||
+            search_result != NULL) {
+                return search_result;
+        }
+
+        cleanup_current_directory_information (search);
+
+        return NULL;
 }
 
 static gboolean
@@ -908,15 +966,15 @@ matches_one_keywords_criterion (const char *criterion,
 
 
 static char *  
-get_root_directory_from_uri (const char *uri)
+unbracket_root_uri (const char *bracketed_uri)
 {
-        char *root_directory;
+        char *root_uri;
         
-        g_return_val_if_fail (strchr (uri, '[') != NULL, NULL);
-        root_directory = strchr (uri, '[') + 1;
-        g_return_val_if_fail (strchr (root_directory, ']') != NULL, NULL);
-        return g_strndup (root_directory, 
-                          strchr (root_directory, ']') - root_directory);
+        g_return_val_if_fail (strchr (bracketed_uri, '[') != NULL, NULL);
+        root_uri = strchr (bracketed_uri, '[') + 1;
+        g_assert (strchr (root_uri, ']') != NULL);
+        return g_strndup (root_uri, 
+                          strchr (root_uri, ']') - root_uri);
        
 }
 
@@ -1048,7 +1106,7 @@ second_word_of (const char *criterion)
         const char *location;
 
         location = criterion;
-        while (!isspace (*location)) {
+        while (!isspace ((guchar) *location)) {
                 location++;
         }
 
@@ -1062,23 +1120,9 @@ last_word_of (const char *criterion)
         const char *location;
 
         location = criterion + strlen (criterion);
-        while (!isspace (*location)) {
+        while (!isspace ((guchar) *location)) {
                 location--;
         }
         return location+1;
 }
 
-
-
-static int
-initialize_results_file_for_writing (char *file_name)
-{
-        return open (file_name, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR | O_SYNC);
-
-} 
-
-static int
-initialize_results_file_for_reading (char *file_name)
-{
-        return open (file_name, O_RDONLY);
-}
