@@ -34,6 +34,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <string.h>
+#include "medusa-log.h"
 #include "medusa-utils.h"
 
 #ifdef HAVE_GETMNTINFO
@@ -44,11 +45,13 @@
 # include <sys/mount.h>
 #elif (HAVE_SYS_MNTTAB_H)
 #include <sys/mnttab.h>
-#elif defined (HAVE_GETMNTENT)
+#else
+#if defined (HAVE_GETMNTENT)
 # include <mntent.h>
 #else
 #warning "Can't find a valid mount function type"
 #endif
+#endif
 
 #include <glib.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -57,7 +60,7 @@
 
 #define CDROM_DRIVE_STATUS	0x5326  /* Get tray position, etc. */
 #define CDSL_CURRENT    	((int) (~0U>>1))
-#define STOPLIST_FILE_NAME MEDUSA_PREFIX "/share/medusa/" "file-index-stoplist"
+#define STOPLIST_FILE_NAME MEDUSA_SYSCONFDIR "/file-index-stoplist"
 
 static void                 file_stoplist_initialize                          (void);
 static void                 unsearched_mount_list_initialize                  (void);
@@ -78,7 +81,9 @@ static GHashTable *unsearched_locations = NULL;
 
 const char *mount_type_skip_list[] = {
         "nfs",
+        "afs",
         "shm",
+        "autofs",
         NULL
 };
 
@@ -140,11 +145,17 @@ file_stoplist_initialize  (void)
 {
   
 	FILE *stop_list_handle;
-	char next_file[NAME_MAX];
+	char next_file[1024];
+
+        g_return_if_fail (unsearched_locations != NULL);
 
 	stop_list_handle = fopen (STOPLIST_FILE_NAME, "r");
-	g_return_if_fail (stop_list_handle != NULL);
+        if (stop_list_handle == NULL) {
+                medusa_log_error ("Couldn't find stop list of files and directories to be ignored.  All files and directories on the system will be considered.");
+                return;
+        }
 
+        /* FIXME: WE should not use fscanf */
 	while (fscanf (stop_list_handle, "%s\n", next_file) != EOF) {
                 /* Skip comment lines */
                 if (next_file[0] == '#') {
@@ -166,7 +177,6 @@ unsearched_mount_list_initialize (void)
 #ifdef HAVE_GETMNTINFO
 	struct statfs *fs_list;
 	int fs_count, i;
-        /* FIXME bugzilla.eazel.com 4565: There is probably a leak here */
 	fs_count = getmntinfo (&fs_list, MNT_LOCAL); /* returns the number of FSes. */
 	for (i = 0; i < fs_count; ++i) {
 		if (mount_type_is_in_skip_list (fs_list[i].f_fstypename)) {
@@ -180,11 +190,15 @@ unsearched_mount_list_initialize (void)
                                              GINT_TO_POINTER (1));
                 }
         }
+
 #elif HAVE_SYS_MNTTAB_H
         FILE *mount_file;
-        struct mnttab *mnttab_entry;
-
+        struct mnttab *mnttab_entry = NULL;
+#ifndef HAVE_SYS_MNTTAB_H
         mount_file = setmntent ("/etc/mnttab", "r");
+#else
+        mount_file = fopen ("/etc/mnttab", "r");
+#endif
         while (getmntent (mount_file, mnttab_entry) != 0) {
                 if (mount_type_is_in_skip_list (mnttab_entry->mnt_fstype)) {
                         g_hash_table_insert (unsearched_locations,
@@ -197,14 +211,22 @@ unsearched_mount_list_initialize (void)
                                              GINT_TO_POINTER (1));
                 }
         }
+#ifdef HAVE_ENDMNTENT
         endmntent (mount_file);
+#else
+        fclose (mount_file);
+#endif;
 
 #elif defined(HAVE_GETMNTENT)
         FILE *mount_file;
         struct mntent *mount_entry;
         
         mount_file = setmntent (MOUNTED, "r");
+#ifndef HAVE_SYS_MNTTAB_H
         while ((mount_entry = getmntent (mount_file)) != NULL) {
+#else
+        while ( getmntent (mount_file,mount_entry) != NULL) {
+#endif
                 if (mount_type_is_in_skip_list (mount_entry->mnt_type)) {
                         g_hash_table_insert (unsearched_locations,
                                              gnome_vfs_get_uri_from_local_path (mount_entry->mnt_dir),
